#ifndef SEGLOC_CONST_H
#define SEGLOC_CONST_H

#ifdef CLASSIFICATION
#define SEGLOC_TICS_PERIOD      40   /* 400 ms (1 tic  = 10 ms) */
#else
#  ifdef H2
/* small period, otherwise pom can not take the pos into account...  */
#  define SEGLOC_TICS_PERIOD      200   /* 500 ms (1 tic  = 5 ms) */

/* Periode (1 tic  = 10 ms) */
#  elif defined(XR4000)
#  define SEGLOC_TICS_PERIOD      20   /* 1000 ms  (1 tic  = 10 ms) */
#  elif defined(ATRV)
#  define SEGLOC_TICS_PERIOD      20   /* 1000 ms  (1 tic  = 10 ms) */
#  elif defined(B21R) || defined(JIDO)
#  define SEGLOC_TICS_PERIOD      20   /* 1000 ms  (1 tic  = 10 ms) */
#  elif defined(SIM)
#  define SEGLOC_TICS_PERIOD      20   /* 1000 ms  (1 tic  = 10 ms) */
#  else 
#  define SEGLOC_TICS_PERIOD      20   /* 1000 ms  (1 tic  = 10 ms) */
#  endif
#endif

/**
 ** SEGMENTATION default params 
 **/
/* cf conf_sg_host in segkit/segmentation/lasegm.c */
#define SEGLOC_DEFAULT_SEG_LEAST_SQUARE_THRES   0.009
#define SEGLOC_DEFAULT_SEG_DIST_MAX_PTS         0.2
#define SEGLOC_DEFAULT_SEG_NB_MIN_PTS           6 

/**
 ** LOCALISATION default params 
 **/

/* Mahalanobis dist test to merge positions */
#ifdef CLASSIFICATION
#define SEGLOC_DEFAULT_LOC_POS_DIST_MAHA_THRES       10000 /* 20 */
#define SEGLOC_DEFAULT_FACT_UNCERTAINTY_TEST         10000
#else
#define SEGLOC_DEFAULT_LOC_POS_DIST_MAHA_THRES       30 /* 20 */
#define SEGLOC_DEFAULT_LOC_POS_DIST_MAHA_THRES_FORCE      20 
#define SEGLOC_DEFAULT_FACT_UNCERTAINTY_TEST          2
#endif

/* Mahalanobis dist test to merge positions */
#define SEGLOC_DEFAULT_LOC_MAP_DIST_MAHA_THRES       1 /* 20 */
#define SEGLOC_DEFAULT_LOC_MAP_DIST_MAHA_THRES_FORCE       12 

/* Angle uncertainty */
#ifdef CLASSIFICATION
#define SEGLOC_DEFAULT_LOC_ANGLE_UNCERTAINTY_DEG 90
#else
#define SEGLOC_DEFAULT_LOC_ANGLE_UNCERTAINTY_DEG 30
#endif

/* Max successive loc failures (loop mode) */
#define SEGLOC_DEFAULT_LOC_NB_MAX_FAILURES  20000


/* Jump threshold (relatively to odometry) for ReLoc request */
#define SEGLOC_X_RELOC_THRESHOLD 1.0
#define SEGLOC_Y_RELOC_THRESHOLD 1.0
#define SEGLOC_THETA_RELOC_THRESHOLD 0.5


/* To specify the quality of the localisation */
#define SEGLOC_X_UNCERTAINTY_THRESHOLD 0.2
#define SEGLOC_Y_UNCERTAINTY_THRESHOLD 0.2
#define SEGLOC_THETA_UNCERTAINTY_THRESHOLD 0.2


/* POM */
#define SEGLOC_FRAME "Sick"

/**
 ** MAP 
 **/

#if 1
#define LAAS_ENVIRONMENT
#define SEGLOC_DEFAULT_ENV "laas-small"

#else

#define SPACE_CITY_ENVIRONMENT
#define SEGLOC_DEFAULT_ENV "tsiolkovski"
#endif

#define SEGLOC_OFFLINE_FILENUM 0

#define XR4000_COMPASS_ENVIRONMENT_NORTH_DEG 145
#define XR4000_COMPASS_ENVIRONMENT_PRECISION_DEG 15

/**
 ** PARTIAL LOC
 **/

/* Allows or not partial localisation ? */
#define SEGLOC_DEFAULT_CORRIDOR_MODE_ALLOWED SEGLOC_FALSE /* for space city *//* SEGLOC_TRUE for LAAS */

/**
 ** PROBABILISTIC ODO
 **/

/* Segkit emulates probabilistic odo ?
   or used the one computed by the locomotion module ? */
#define SEGLOC_DEFAULT_ERR_SIMUL SEGLOC_TRUE
/* Parameters if probabilistic odo emulated by segloc */
#define SEGLOC_DEFAULT_ERR_SIMUL_LIN 0.1       /* 0.1  to 0.01 */
#define SEGLOC_DEFAULT_ERR_SIMUL_ANG 0.1       /* 0.15 to 0.01 */
#define SEGLOC_DEFAULT_ERR_SIMUL_LIN_ANG 0.01  /* 0.1  to 0.001 */

/**
 ** OFFLINE
 **/
#ifdef CLASSIFICATION
#define SEGLOC_OFFLINE_MODE TRUE
#else
#define SEGLOC_OFFLINE_MODE FALSE
#endif


#endif /* SEGLOC_CONST_H */
