#ifndef SEGLOC_STRUCT_H
#define SEGLOC_STRUCT_H

#include <pomStruct.h>

/**----------------------------------------------------------------------
 **  COMMON USEFULL STRUCT
 **/

/* Usefull flag */
typedef enum SEGLOC_BOOL {
  SEGLOC_FALSE,
  SEGLOC_TRUE
} SEGLOC_BOOL;

#define SEGLOC_MAX_STRLEN 256

typedef struct SEGLOC_FILE_PARAMS {
  char baseName[SEGLOC_MAX_STRLEN]; /* base directory */
  int num;			    /* -1 (no num extension) or first image 
				       number (auto-incremented in save mode) */
  int unused;
} SEGLOC_FILE_PARAMS;

typedef struct SEGLOC_INIT {
  SEGLOC_FILE_PARAMS mapFile;
  char posPosterName[64];
  char laserDataPosterName[64];
} SEGLOC_INIT;

typedef enum SEGLOC_CLIP_MODE {
  SEGLOC_REMOVE_IN,
  SEGLOC_REMOVE_OUT
} SEGLOC_CLIP_MODE;

typedef struct SEGLOC_CLIP_MAP_PARAMS {
  SEGLOC_CLIP_MODE clipMode;
  int unused;
  double xmin,  ymin,  xmax,  ymax;
} SEGLOC_CLIP_MAP_PARAMS;

typedef struct SEGLOC_POS {
  double x, y, theta;
} SEGLOC_POS;

typedef struct SEGLOC_UNC {
  double ux, uy, utheta;
} SEGLOC_UNC;

typedef struct SEGLOC_SPEED {
  double v, w;
} SEGLOC_SPEED;

typedef struct SEGLOC_POS_SPEED {
  SEGLOC_POS pos;
  SEGLOC_SPEED speed;
} SEGLOC_POS_SPEED;

typedef struct SEGLOC_VARIANCES {
  double vxx, vyy, vtt, vxy, vxt, vyt;
} SEGLOC_VARIANCES;

typedef struct SEGLOC_POS_VAR {
  SEGLOC_POS pos;
  SEGLOC_VARIANCES var;
} SEGLOC_POS_VAR;

typedef struct SEGLOC_POS_VAR_SPEED {
  SEGLOC_POS pos;
  SEGLOC_VARIANCES var;
  SEGLOC_SPEED speed;
} SEGLOC_POS_VAR_SPEED;

typedef struct SEGLOC_UNCERTAIN_POS {
  SEGLOC_POS pos;
  SEGLOC_UNC unc;
} SEGLOC_UNCERTAIN_POS;

typedef struct SEGLOC_START_END_POS {
  SEGLOC_POS start;
  SEGLOC_POS end;
} SEGLOC_START_END_POS;

/**----------------------------------------------------------------------
 **  INIT
 **/
/* typedef struct SEGLOC_INIT { */
/*   char envName[SEGLOC_MAX_STRLEN]; */
/*   SEGLOC_BOOL offlineOn; */
/*   int unused; */
/* } SEGLOC_INIT; */
 
/**----------------------------------------------------------------------
 **  LOCALISATION DATA
 **/

/* The different localisation modes */
#define SEGLOC_NB_LOC_MODES 4
typedef enum SEGLOC_LOC_MODE {
  SEGLOC_LOC_LOCAL,
  SEGLOC_FORCE_LOC_LOCAL,
  SEGLOC_LOC_ONCE,
  SEGLOC_LOC_LOOP
} SEGLOC_LOC_MODE;

/* Update or not loco (not used now) */
typedef enum SEGLOC_UPDATE_LOCO {
  SEGLOC_NO_UPDATE,   /* No update */
  SEGLOC_RESET_VAR,   /* Reset to a default value the odo variances */
  SEGLOC_UPDATE_VAR,  /* Update variances according to mapserver var */
  SEGLOC_UPDATE_POS   /* Update pos and var according to mapserver data */
} SEGLOC_UPDATE_LOCO;

/* Localisation parameters */
typedef struct SEGLOC_LOC_PARAMS {
  SEGLOC_LOC_MODE    locMode;          /* local, once or loop */
  int                nbMaxFailures;
  SEGLOC_BOOL        variancesFilter;  /* Filter according to loco variances 
					  (used posDistMahaThres) */
  SEGLOC_BOOL        visibilityFilter; /* Filter according visibility */
  SEGLOC_BOOL        useCompass;
  SEGLOC_UPDATE_LOCO updatePosMode;    /* loco pos updated ? */
  long               posDistMahaThres; /* if variancesFilter is TRUE */
  long               mapDistMahaThres; /* for MapTrData only 
					  (if first loc failed) */
  double             angleUncertainty; /* confidance angle in rd (PI: unkown)*/
  double             factUncertaintyTest;
  SEGLOC_BOOL        segkitSimulErr;
  SEGLOC_BOOL        corridorModeAllowed;
  double simulErrLin;
  double simulErrAng;
  double simulErrLinAng;
}SEGLOC_LOC_PARAMS;


/* Localisation quality */
typedef enum SEGLOC_LOC_QUALITY {
  SEGLOC_LOC_QUALITY_OK,      /* Loc OK */
  SEGLOC_LOC_QUALITY_MEDIUM,  /* Partial loc (ie, corridor) */
  SEGLOC_LOC_QUALITY_BAD,     /* No loc but uncertainty still OK */
  SEGLOC_LOC_QUALITY_KO       /* Lost */
}SEGLOC_LOC_QUALITY;

/* Localisation confidence */
typedef enum SEGLOC_CONFIDENCE_ATTRIBUTES {
  SEGLOC_CONF_NB_MATCH,           /* nb matching segs / nb acq segs */
  SEGLOC_CONF_LENGTH_MATCH,       /* matching length / acq length */
  SEGLOC_CONF_ANGLE_MATCH,        /* matching angle from robot view point /
				     acq angle from robot view point */
  SEGLOC_CONF_LENGTH_NOT_MATCH,  /* not matching length / acq length */
  SEGLOC_CONF_LENGTH_PERCENT_NOT_MATCH, /* same in percentage */
  SEGLOC_CONF_W,                 /* angular speed */
  SEGLOC_CONF_POS_UNC,           /* Position uncertainty */
  SEGLOC_CONF_ANGLE_UNC,         /* Angle uncertainty */
  SEGLOC_NB_CONF_ATTRIBUTES
} SEGLOC_CONFIDENCE_ATTRIBUTES;

/* Weigth  to compute the confidence from the attributes 
   !! Only computed if quality==SEGLOC_LOC_QUALITY_OK !! */
/* #define SEGLOC_CONFIDENCE_ATT_WEIGTH \ */
/* {0.44, 0.56, 0.55, -0.16, -0.31, -0.02, -0.12, -0.16} */
#define SEGLOC_CONFIDENCE_ATT_WEIGTH \
{0.44, 0.56, 0.55, -0.16, -0.31, -0.02, 0, 0}

#define SEGLOC_DEFAULT_CONFIDENCE_FLAGS { \
  SEGLOC_TRUE, SEGLOC_TRUE, SEGLOC_TRUE, SEGLOC_TRUE, \
  SEGLOC_TRUE, SEGLOC_TRUE, SEGLOC_TRUE, SEGLOC_TRUE}

/* Localisation results */
typedef struct SEGLOC_LOC_REPORT {
  SEGLOC_LOC_QUALITY quality;
  int nbPossibilities;
  int nbSuccesses;
  int nbMatches;
    /* Following confidances only computed for the case :
       quality == SEGLOC_LOC_QUALITY_OK !! 
    XXX - It is just an attribute, not a probability,  XXX
    XXX   computed form the "analyse en composantes    XXX
    XXX   principales" by fakhfak                      XXX
    XXX - Can be also computed in corridors            XXX */
  double confidence; 
} SEGLOC_LOC_REPORT;

typedef struct SEGLOC_TIMES {
  unsigned int sickTime;   /* Date of sick acquisition 
			      (warning: may be not the same clock,
			      however usefull to identify sick acq) */
  unsigned int transfUpdateTime;  /* Date of last successfull reloc */
  unsigned int lastlocTime;       /* Date of last computed loc 
				   (one can correct position even if
				   no new transform has been founded) */
  int unused;
} SEGLOC_TIMES;


typedef enum SEGLOC_SURV_LOC_REPORT {
  SEGLOC_X_OUT,       /* x uncertainty out of threshold */
  SEGLOC_Y_OUT,       /* y uncertainty out of threshold */
  SEGLOC_THETA_OUT,   /* theta uncertainty out of threshold */
  SEGLOC_X_OR_Y_OUT
} SEGLOC_SURV_LOC_REPORT;


/**----------------------------------------------------------------------
 ** MODULE FUNCTIONAL STATE
 **/

/* Functional state of the module */
typedef struct SEGLOC_STATE {
  SEGLOC_BOOL     mapLoaded;       /* Just to know if there is a map */
  SEGLOC_BOOL     segmentationOn;  /* Segmentation activated or not */
  SEGLOC_BOOL     localisationOn;  /* Localisation activated or not 
				      (if actived and not segmentation, 
				      then segmentation temporarily activated) */
  SEGLOC_LOC_MODE locMode;         /* once, local or loop */
  int             bilan;
} SEGLOC_STATE;


typedef struct SEGLOC_STAT {
  int nbLocOK;
  int nbPartialLoc;
  int nbLocFailed;
  int nbLocCrashed;
} SEGLOC_STAT;


/**----------------------------------------------------------------------
 ** OFF-LINE RUNS
 **/

typedef struct SEGLOC_OFFLINE {
  SEGLOC_BOOL offlineOn;
  SEGLOC_BOOL setSegmentationOff;   /* Desactive la segmentation continue
				       (evite de dfiler les fichiers pour rien
				       tant que pas de loc ou de requete 
				       explicite a SegmentationOn) */
  char baseName[SEGLOC_MAX_STRLEN];  /* base directory */
  int fileNum;			   /* first image number (auto-incremented)*/
  SEGLOC_BOOL readPrevClassif;
} SEGLOC_OFFLINE;

/**----------------------------------------------------------------------
 ** COMPASS
 **/

#define SEGLOC_NB_MAX_COMPASS 100
typedef struct SEGLOC_COMPASS_ENV {
  double environmentNorthDeg;
  double environmentAccuracyDeg;
} SEGLOC_COMPASS_ENV;

typedef struct SEGLOC_COMPASS_RES {
  int nbData;
  int unused;
  double rawAngleDeg;           /* Valeur brute compas */
  double correctedAngleDeg;     /* Angle dans le repere carte */
  double angleUncDeg;           /* Incertitude */
  double deltaAngleAverageDeg;  /* Ecart angulaire moyen segloc-compas */
  double deltaAngleSquareAverageRad;  /* Ecart angulaire moyen segloc-compas */
  double deltaAngle3sigmaDeg; 
  double deltaAngleMinDeg;
  double deltaAngleMaxDeg;
} SEGLOC_COMPASS_RES;

typedef struct SEGLOC_COMPASS_STR {
    SEGLOC_COMPASS_ENV compassEnv;
    SEGLOC_COMPASS_RES compassRes;
} SEGLOC_COMPASS_STR;

/**----------------------------------------------------------------------
 ** MAP EDITION
 **/


/**------------------------------
 ** DESCRIPTION OF SEGMENTS
 **/

typedef struct SEGLOC_SEG {
  double x1, y1;
  double x2, y2;
} SEGLOC_SEG;

#ifdef SEG_VARIANCES
typedef struct SEGLOC_SEG_VAR {
  double r, o;
  double vr, vo, vro;
} SEGLOC_SEG_VAR;
#endif /* SEG_VARIANCES */


typedef struct SEGLOC_2SEGS {
  SEGLOC_SEG  seg1;
  SEGLOC_SEG  seg2;
} SEGLOC_2SEGS;

typedef struct SEGLOC_CUT_SEG {
  SEGLOC_SEG  seg;
  double l1;
  double l2;
} SEGLOC_CUT_SEG;

typedef struct SEGLOC_MAP_EDIT {
  SEGLOC_2SEGS segs;
  SEGLOC_CUT_SEG  segCut;
} SEGLOC_MAP_EDIT;

/*-- POSTER STRUCTURES ------------------------------------------------- */

/**----------------------------
 ** SEGMENTATION PARAMS
 **/

/* Parameters for segmentation (not used now) */
typedef struct SEGLOC_SEG_PARAMS {
  SEGLOC_BOOL getVariances;
  int nbMinSegPts;          /* Nb min pts in a seg */
  double leastSquareThres;  /* Threshold for the least square estimation */
  double distMaxSegPts;     /* Max dist between 2 points in a seg */
} SEGLOC_SEG_PARAMS;


/**------------------------------
 ** DESCRIPTION OF SEGMENTS
 **/

typedef struct SEGLOC_SEG_HEADER {
  int nbSegs;
  SEGLOC_BOOL variancesUpdated;  /* Segment variances recorded or not */
  SEGLOC_POS  locoPos;           /* Robot pos during sick acquisition */
  unsigned int sickTime;         /* Acquisition time */
  int nbMatchSegs;               /* Some more segs related to the loc */
  SEGLOC_VARIANCES variances;
  POM_SENSOR_POS sickPomPos;     /* Robot/sick pos during sick acquisition */
} SEGLOC_SEG_HEADER;

/* Acq segs poster */
#define SEGLOC_MAX_ACQ_SEGS 100

typedef struct SEGLOC_ACQ_SEGS {
  SEGLOC_SEG_HEADER header;
  SEGLOC_SEG segs[SEGLOC_MAX_ACQ_SEGS];
#ifdef SEG_VARIANCES
  SEGLOC_SEG_VAR var[SEGLOC_MAX_ACQ_SEGS];
#endif /* SEG_VARIANCES */
} SEGLOC_ACQ_SEGS;

/* Aspect segs poster */
#ifdef CLASSIFICATION
#define SEGLOC_MAX_ASPECT_SEGS 300
#else
#define SEGLOC_MAX_ASPECT_SEGS 100
#endif

typedef struct SEGLOC_ASPECT_SEGS {
  SEGLOC_SEG_HEADER header;
  SEGLOC_SEG segs[SEGLOC_MAX_ASPECT_SEGS];
} SEGLOC_ASPECT_SEGS;


/**------------------------------
 ** DESCRIPTION OF MATCHED SEGMENTS
 **/

typedef struct SEGLOC_MATCHED_SEG_HEADER {
  int nbMatchSegs;               /* Some more segs related to the loc */
  unsigned int sickTime;         /* Acquisition time */
  SEGLOC_POS  seglocPos;           /* segloc pos during sick acquisition */
  SEGLOC_VARIANCES variances;
} SEGLOC_MATCHED_SEG_HEADER;


typedef struct SEGLOC_MATCHED_SEG {
  SEGLOC_SEG acqSeg;
  int mapSeg;        /* XXX should not be the aspectSeg ? */
  int unused;
} SEGLOC_MATCHED_SEG;

typedef struct SEGLOC_MATCHED_SEGS {
  SEGLOC_MATCHED_SEG_HEADER header;
  SEGLOC_MATCHED_SEG segs[SEGLOC_MAX_ACQ_SEGS];
} SEGLOC_MATCHED_SEGS;



/**------------------------------
 ** DESCRIPTION OF MAP
 **/

/** Map Poster **/
#define SEGLOC_MAX_MAP_SEGS 1000
#define SEGLOC_ENV_NAME_LENGTH 64

typedef struct SEGLOC_MAP_HEADER {
  int nbSegs;
  int mapNum;
  SEGLOC_POS  pos;              /* Robot pos during sick acquisition */
  char envName[SEGLOC_ENV_NAME_LENGTH];
} SEGLOC_MAP_HEADER;

typedef struct SEGLOC_MAP {
  SEGLOC_MAP_HEADER header;
  SEGLOC_SEG segs[SEGLOC_MAX_MAP_SEGS];
#ifdef SEG_VARIANCES
  SEGLOC_SEG_VAR var[SEGLOC_MAX_MAP_SEGS];
#endif /* SEG_VARIANCES */
} SEGLOC_MAP;

#endif /* SEGLOC_STRUCT_H */
